@@ -18,6 +18,14 @@
 #'   frames must have the same number of rows. To match by value, not
 #'   position, see \code{left_join} etc. When row-binding, columns are
 #'   matched by name, and any values that don't match will be filled with NA.
+#' @param .id Data frames identifier.
+#'
+#'   When \code{.id} is supplied, a new column of identifiers is
+#'   created to link each row to its original data frame. The labels
+#'   are taken from the named arguments to \code{bind_rows()}. When a
+#'   list of data frames is supplied, the labels are taken from the
+#'   names of the list. If no names are found a numeric sequence is
+#'   used instead.
 #' @return \code{bind_rows} and \code{bind_cols} always return a \code{tbl_df}
 #' @aliases rbind_all rbind_list
 #' @examples
@@ -30,6 +38,12 @@
 #' bind_rows(list(one, two))
 #' bind_rows(split(mtcars, mtcars$cyl))
 #'
+#' # When you supply a column name with the `.id` argument, a new
+#' # column is created to link each row to its original data frame
+#' bind_rows(list(one, two), .id = "id")
+#' bind_rows(list(a = one, b = two), .id = "id")
+#' bind_rows("group 1" = one, "group 2" = two, .id = "groups")
+#'
 #' # Columns don't need to match when row-binding
 #' bind_rows(data.frame(x = 1:3), data.frame(y = 1:4))
 #' \dontrun{
@@ -54,18 +68,31 @@ NULL
 
 #' @export
 #' @rdname bind
-bind_rows <- function(x, ...) {
-  if (is.list(x) && !is.data.frame(x)) {
-    rbind_all(x)
-  } else {
-    rbind_all(list(x, ...))
+bind_rows <- function(..., .id = NULL) {
+  dots <- list(...)
+  if (is.list(dots[[1]]) &&
+        !is.data.frame(dots[[1]]) &&
+        !length(dots[-1])) {
+    x <- dots[[1]]
   }
+  else {
+    x <- dots
+  }
+
+  if (!is.null(.id)) {
+    if (!(is.character(.id) && length(.id) == 1)) {
+      stop(".id is not a string", call. = FALSE)
+    }
+    names(x) <- names(x) %||% seq_along(x)
+  }
+
+  rbind_all(x, .id)
 }
 
 #' @export
 #' @rdname bind
 bind_cols <- function(x, ...) {
-  if (is.list(x) && !is.data.frame(x)) {
+  if (is.list(x) && !is.data.frame(x) && !length(list(...)) ) {
     cbind_all(x)
   } else {
     cbind_all(list(x, ...))
