@@ -6,7 +6,7 @@
 #'   print \code{dplyr.print_min}
 #' @param width Width of text output to generate. This defaults to NULL, which
 #'   means use \code{getOption("width")} and only display the columns that
-#'   fit on one screen. You can also set \code{option(dplyr.width = Inf)} to
+#'   fit on one screen. You can also set \code{options(dplyr.width = Inf)} to
 #'   override this default and always print all columns.
 #' @keywords internal
 #' @examples
@@ -25,7 +25,7 @@ NULL
 #' @rdname dplyr-formatting
 dim_desc <- function(x) {
   d <- dim(x)
-  d2 <- format(d, big.mark = ",", justify = "none", trim = TRUE)
+  d2 <- big_mark(d)
   d2[is.na(d)] <- "??"
 
   paste0("[", paste0(d2, collapse = " x "), "]")
@@ -59,14 +59,22 @@ trunc_mat <- function(x, n = NULL, width = NULL) {
   df[is_list] <- lapply(df[is_list], function(x) vapply(x, obj_type, character(1)))
 
   mat <- format(df, justify = "left")
-
-  width <- width %||% getOption("dplyr.width", NULL) %||% getOption("width")
-
   values <- c(format(rownames(mat))[[1]], unlist(mat[1, ]))
+
+  classes <- paste0("(", vapply(df, type_sum, character(1)), ")")
   names <- c("", colnames(mat))
-  w <- pmax(nchar(values), nchar(names))
+
+  # Column needs to be as wide as widest of name, values, and class
+  w <- pmax(
+    pmax(
+      nchar(encodeString(values)),
+      nchar(encodeString(names))
+    ),
+    nchar(encodeString(c("", classes)))
+  )
   cumw <- cumsum(w + 1)
 
+  width <- width %||% getOption("dplyr.width", NULL) %||% getOption("width")
   too_wide <- cumw[-1] > width
   # Always display at least one column
   if (all(too_wide)) {
@@ -74,6 +82,8 @@ trunc_mat <- function(x, n = NULL, width = NULL) {
     df[[1]] <- substr(df[[1]], 1, width)
   }
   shrunk <- format(df[, !too_wide, drop = FALSE])
+  shrunk <- rbind(" " = classes, shrunk)
+  colnames(shrunk) <- colnames(df)[!too_wide]
 
   needs_dots <- is.na(rows) || rows > n
   if (needs_dots) {
@@ -159,7 +169,7 @@ obj_type.default <- function(x) {
   } else if (!isS4(x)) {
     paste0("<S3:", paste0(class(x), collapse = ", "), ">")
   } else {
-    paste0("<S4:", paste0(is(x), collapse = ", "), ">")
+    paste0("<S4:", paste0(methods::is(x), collapse = ", "), ">")
   }
 }
 
@@ -171,3 +181,10 @@ obj_type.data.frame <- function(x) {
 obj_type.data_frame <- function(x) {
   paste0("<data_frame [", paste0(dim(x), collapse = ","), "]", ">")
 }
+
+# function for the thousand separator,
+# returns "," unless it's used for the decimal point, in which case returns "."
+big_mark <- function(x, ...) {
+  mark <- if (identical(getOption("OutDec"), ",")) "." else ","
+  formatC(x, big.mark = mark, ...)
+}
