@@ -7,11 +7,10 @@
 #include <dplyr/workarounds.h>
 
 using namespace Rcpp ;
-#include <Rcpp/Benchmark/Timer.h>
 #include <tools/all_na.h>
 // borrowed from Rcpp11
 #ifndef RCPP_DEBUG_OBJECT
-    #define RCPP_DEBUG_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( "str"), OBJ ), R_GlobalEnv ) ) ;    
+    #define RCPP_DEBUG_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( "str"), OBJ ), R_GlobalEnv ) ) ;
 #endif
 
 #ifndef RCPP_INSPECT_OBJECT
@@ -19,6 +18,7 @@ using namespace Rcpp ;
 #endif
 
 #include <boost/scoped_ptr.hpp>
+#include <boost/shared_ptr.hpp>
 #include <boost/functional/hash.hpp>
 
 #ifndef dplyr_hash_map
@@ -44,28 +44,24 @@ using namespace Rcpp ;
 
 namespace dplyr {
     class Result ;
-    class ResultSet ;   
+    class ResultSet ;
     class Reducer_Proxy ;
     class DataFrameVisitors ;
     class DataFrameJoinVisitors ;
     class LazySubsets ;
-    template <typename OUT, int INPUT_RTYPE> class Reducer ; 
     std::string get_single_class(SEXP x) ;
-    
+
     template <typename Index>
     DataFrame subset( DataFrame df, const Index& indices, CharacterVector classes) ;
-    
+
 }
 dplyr::Result* get_handler( SEXP, const dplyr::LazySubsets&, const Environment& ) ;
 bool can_simplify(SEXP) ;
 
 void assert_all_white_list(const DataFrame&) ;
-inline SEXP as_symbol(SEXP x) {
-    return Rf_install( CHAR(x) );
-}
 inline SEXP shared_SEXP(SEXP x){
-    SET_NAMED(x, 2 );  
-    return x ;  
+    SET_NAMED(x, 2 );
+    return x ;
 }
 void check_supported_type(SEXP) ;
 
@@ -81,7 +77,7 @@ inline SEXP pairlist_shallow_copy(SEXP p){
         SET_TAG(q, TAG(p)) ;
         p = CDR(p) ;
     }
-    return attr ;   
+    return attr ;
 }
 
 inline void copy_attributes(SEXP out, SEXP data){
@@ -97,7 +93,7 @@ inline void copy_most_attributes(SEXP out, SEXP data){
     copy_attributes(out,data) ;
     Rf_setAttrib( out, R_NamesSymbol, R_NilValue ) ;
 }
-    
+
 CharacterVector dfloc(List) ;
 SEXP shallow_copy(const List& data) ;
 
@@ -110,6 +106,8 @@ typedef dplyr::Result* (*HybridHandler)(SEXP, const dplyr::LazySubsets&, int) ;
     SEXP get_date_classes() ;
 #endif
 
+#include <dplyr/DataFrameAble.h>
+#include <dplyr/CharacterVectorOrderer.h>
 #include <dplyr/white_list.h>
 #include <dplyr/check_supported_type.h>
 #include <dplyr/visitor_set/visitor_set.h>
@@ -125,11 +123,17 @@ typedef dplyr::Result* (*HybridHandler)(SEXP, const dplyr::LazySubsets&, int) ;
 #include <dplyr/comparisons.h>
 #include <dplyr/comparisons_different.h>
 #include <dplyr/VectorVisitor.h>
+#include <dplyr/SubsetVectorVisitor.h>
 #include <dplyr/OrderVisitor.h>
 #include <dplyr/VectorVisitorImpl.h>
+#include <dplyr/SubsetVectorVisitorImpl.h>
 #include <dplyr/DataFrameVisitors.h>
+#include <dplyr/DataFrameSubsetVisitors.h>
+#include <dplyr/DataFrameColumnSubsetVisitor.h>
+#include <dplyr/MatrixColumnSubsetVectorVisitor.h>
 #include <dplyr/MatrixColumnVisitor.h>
 #include <dplyr/DataFrameColumnVisitor.h>
+#include <dplyr/subset_visitor.h>
 #include <dplyr/visitor.h>
 #include <dplyr/OrderVisitorImpl.h>
 #include <dplyr/JoinVisitor.h>
@@ -188,7 +192,7 @@ SEXP strip_group_attributes(Data df){
 template <typename T>
 CharacterVector names( const T& obj ){
     SEXP x = obj ;
-    return Rf_getAttrib(x, Rf_install("names" ) ) ;    
+    return Rf_getAttrib(x, Rf_install("names" ) ) ;
 }
 
 
