@@ -5,7 +5,9 @@ namespace dplyr {
        
     class CallElementProxy {
     public:
-        CallElementProxy(SEXP symbol_, SEXP object_) : symbol(symbol_), object(object_){}
+        CallElementProxy(SEXP symbol_, SEXP object_) : 
+            symbol(symbol_), object(object_)
+        {}
         
         inline void set(SEXP value){ 
             SETCAR(object, value) ;
