@@ -6,8 +6,12 @@ namespace dplyr {
     template <int RTYPE, bool NA_RM>
     class Sd : public Processor<REALSXP, Sd<RTYPE,NA_RM> > {
     public:
+        typedef Processor<REALSXP, Sd<RTYPE,NA_RM> > Base ;
         
-        Sd(SEXP x, bool is_summary = false) : var(x, is_summary) {}
+        Sd(SEXP x, bool is_summary = false) : 
+            Base(x), 
+            var(x, is_summary) 
+        {}
         ~Sd(){}
         
         inline double process_chunk( const SlicingIndex& indices ){
