@@ -6,9 +6,13 @@ namespace dplyr {
     template <int RTYPE, bool NA_RM>
     class Max : public Processor<RTYPE, Max<RTYPE,NA_RM> > {
     public:
+        typedef Processor<RTYPE, Max<RTYPE,NA_RM> > Base ;
         typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
         
-        Max(SEXP x, bool is_summary_ = false) : data_ptr( Rcpp::internal::r_vector_start<RTYPE>(x) ), is_summary(is_summary_) {}
+        Max(SEXP x, bool is_summary_ = false) :
+            Base(x), 
+            data_ptr( Rcpp::internal::r_vector_start<RTYPE>(x) ), 
+            is_summary(is_summary_) {}
         ~Max(){}
         
         STORAGE process_chunk( const SlicingIndex& indices ){
@@ -39,9 +43,13 @@ namespace dplyr {
     template <int RTYPE>
     class Max<RTYPE,false> : public Processor<RTYPE, Max<RTYPE,false> > {
     public:
+        typedef Processor<RTYPE, Max<RTYPE,false> > Base ;
         typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
         
-        Max(SEXP x, bool is_summary_ = false) : data_ptr( Rcpp::internal::r_vector_start<RTYPE>(x) ), is_summary(is_summary_)  {}
+        Max(SEXP x, bool is_summary_ = false) :
+            Base(x), 
+            data_ptr( Rcpp::internal::r_vector_start<RTYPE>(x) ), 
+            is_summary(is_summary_)  {}
         ~Max(){}
         
         STORAGE process_chunk( const SlicingIndex& indices ){
