@@ -4,33 +4,36 @@ using namespace Rcpp ;
 using namespace dplyr ;
 
 template <typename Dots>
-List rbind__impl( Dots dots ){
+List rbind__impl( Dots dots, SEXP id = R_NilValue ){
     int ndata = dots.size() ;
     int n = 0 ;
+    std::vector<DataFrameAble> chunks ;
+    std::vector<int> df_nrows ;
+
     for( int i=0; i<ndata; i++) {
-      DataFrame df = dots[i] ;
-      CharacterVector df_names = df.names() ;
-      if( any(is_na(df_names)).is_true() ){
-        stop( "corrupt data frame at index %d", (i+1) ) ;
-      }
-      if( df.size() ) n += df.nrows() ;
+      chunks.push_back( DataFrameAble( dots[i] ) ) ;
+
+      int nrows = chunks[i].nrows() ;
+      df_nrows.push_back(nrows) ;
+      n += nrows ;
     }
     pointer_vector<Collecter> columns ;
 
     std::vector<String> names ;
     int k=0 ;
+
+    Function enc2native( "enc2native" ) ;
     for( int i=0; i<ndata; i++){
         Rcpp::checkUserInterrupt() ;
 
-        DataFrame df = dots[i] ;
+        const DataFrameAble& df = chunks[i] ;
         if( !df.size() ) continue ;
 
-        DataFrameVisitors visitors( df, df.names() ) ;
         int nrows = df.nrows() ;
 
-        CharacterVector df_names = df.names() ;
+        CharacterVector df_names = enc2native(df.names()) ;
         for( int j=0; j<df.size(); j++){
-            SEXP source = df[j] ;
+            SEXP source = df.get(j) ;
             String name = df_names[j] ;
 
             Collecter* coll = 0;
@@ -46,7 +49,6 @@ List rbind__impl( Dots dots ){
                 columns.push_back( coll );
                 names.push_back(name) ;
             }
-
             if( coll->compatible(source) ){
                 // if the current source is compatible, collect
                 coll->collect( SlicingIndex( k, nrows), source ) ;
@@ -86,27 +88,44 @@ List rbind__impl( Dots dots ){
     }
 
     int nc = columns.size() ;
-    List out(nc) ;
-    CharacterVector out_names(nc) ;
-    for( int i=0; i<nc; i++){
-        out[i] = columns[i]->get() ;
-        out_names[i] = names[i] ;
-    }
-    out.attr( "names" ) = out_names ;
-    set_rownames( out, n );
-    out.attr( "class" ) = classes_not_grouped() ;
+    int has_id = Rf_isNull(id) ? 0 : 1;
 
+    List out(nc + has_id) ;
+    CharacterVector out_names(nc + has_id) ;
+    for( int i=0; i<nc; i++){
+        out[i + has_id] = columns[i]->get() ;
+        out_names[i + has_id] = names[i] ;
+    }
+
+    // Add vector of identifiers if .id is supplied
+    if (!Rf_isNull(id)) {
+      CharacterVector df_names = dots.names() ;
+      CharacterVector id_col = no_init(n) ;
+
+      CharacterVector::iterator it = id_col.begin() ;
+      for (int i=0; i<ndata; ++i) {
+        std::fill( it, it + df_nrows[i], df_names[i] ) ;
+        it += df_nrows[i] ;
+      }
+
+      out[0] = id_col ;
+      out_names[0] = Rcpp::as<std::string>(id) ;
+    }
+
+    out.attr( "names" ) = out_names ;
+    set_rownames( out, n ) ;
+    out.attr( "class" ) = classes_not_grouped() ;
     return out ;
 }
 
 //' @export
 // [[Rcpp::export]]
-List rbind_all( StrictListOf<DataFrame, NULL_or_Is<DataFrame> > dots ){
-    return rbind__impl(dots) ;
+List rbind_all( List dots, SEXP id = R_NilValue ){
+    return rbind__impl(dots, id) ;
 }
 
 // [[Rcpp::export]]
-List rbind_list__impl( DotsOf<DataFrame> dots ){
+List rbind_list__impl( Dots dots ){
     return rbind__impl(dots) ;
 }
 
@@ -114,14 +133,18 @@ template <typename Dots>
 List cbind__impl( Dots dots ){
   int n = dots.size() ;
 
+  std::vector<DataFrameAble> chunks ;
+  for( int i=0; i<n; i++) {
+    chunks.push_back( DataFrameAble( dots[i] ) );
+  }
+
   // first check that the number of rows is the same
-  DataFrame df = dots[0] ;
+  const DataFrameAble& df = chunks[0] ;
   int nrows = df.nrows() ;
   int nv = df.size() ;
   for( int i=1; i<n; i++){
-    DataFrame current = dots[i] ;
+    const DataFrameAble& current = dots[i] ;
     if( current.nrows() != nrows ){
-      std::stringstream ss ;
       stop( "incompatible number of rows (%d, expecting %d)", current.nrows(), nrows ) ;
     }
     nv += current.size() ;
@@ -135,11 +158,11 @@ List cbind__impl( Dots dots ){
   for( int i=0, k=0 ; i<n; i++){
       Rcpp::checkUserInterrupt() ;
 
-      DataFrame current = dots[i] ;
+      const DataFrameAble& current = dots[i] ;
       CharacterVector current_names = current.names() ;
       int nc = current.size() ;
       for( int j=0; j<nc; j++, k++){
-          out[k] = shared_SEXP(current[j]) ;
+          out[k] = shared_SEXP(current.get(j)) ;
           out_names[k] = current_names[j] ;
       }
   }
@@ -150,7 +173,7 @@ List cbind__impl( Dots dots ){
 }
 
 // [[Rcpp::export]]
-List cbind_all( StrictListOf<DataFrame, NULL_or_Is<DataFrame> > dots ){
+List cbind_all( List dots ){
     return cbind__impl( dots ) ;
 }
 
@@ -190,4 +213,3 @@ SEXP combine_all( List data ){
     RObject out = coll->get() ;
     return out ;
 }
-
