@@ -1,7 +1,3 @@
-library(data.table, warn.conflicts = FALSE)
-library(RSQLite, quietly = TRUE)
-library(RPostgreSQL, quietly = TRUE)
-
 int_to_num <- function(x, y) {
   is_integer_x <- vapply(x, is.integer, logical(1))
   is_integer_y <- vapply(y, is.integer, logical(1))
