@@ -195,7 +195,7 @@ test_that("mutate modifies same column repeatedly (#243)", {
 
 test_that("mutate errors when results are not compatible accross groups (#299)",{
   d <- data.frame(x = rep(1:5, each = 3))
-  expect_error(mutate(group_by(d,x),val = ifelse(x < 3, NA, 2)))
+  expect_error(mutate(group_by(d,x),val = ifelse(x < 3, "foo", 2)))
 })
 
 test_that("assignments are forbidden (#315)", {
@@ -304,11 +304,10 @@ test_that("mutate works on zero-row grouped data frame (#596)", {
 
 test_that("Non-ascii column names in version 0.3 are not duplicated (#636)", {
   df  <- data_frame(a = "1", b = "2")
-  names(df) <- c("a", "å")
-  Encoding(names(df)) <- "unknown"
+  names(df) <- c("a", enc2native("\u4e2d"))
 
   res <- df %>% mutate_each(funs(as.numeric)) %>% names
-  expect_equal(res, c("a", "å") )
+  expect_equal(res, names(df))
 })
 
 test_that("nested hybrid functions do the right thing (#637)", {
@@ -362,3 +361,69 @@ test_that("no utf8 invasion (#722)", {
   source("utf-8.R", local = TRUE)
 })
 
+test_that("mutate works on empty data frames (#1142)", {
+  df <- data.frame()
+  res <- df %>% mutate
+  expect_equal( nrow(res), 0L )
+  expect_equal( length(res), 0L )
+
+  res <- df %>% mutate(x = numeric())
+  expect_equal( names(res), "x")
+  expect_equal( nrow(res), 0L )
+  expect_equal( length(res), 1L)
+})
+
+test_that("mutate handles 0 rows rowwise #1300",{
+  a <- data.frame(x= 1)
+  b <- data.frame(y = character(), stringsAsFactors = F)
+
+  g <- function(y){1}
+  f <- function() { b %>% rowwise() %>% mutate(z = g(y))}
+
+  res <- f()
+  expect_equal( nrow(res), 0L )
+
+  expect_error(a %>% mutate(b = f()), "wrong result size" )
+  expect_error(a %>% rowwise() %>% mutate(b = f()), "incompatible size")
+})
+
+test_that("regression test for #637", {
+  res <- mtcars %>% mutate(xx = mean(1))
+  expect_true( all(res$xx == 1))
+
+  res <- mtcars %>% mutate(xx = sum(mean(mpg)))
+  expect_true( all( res$xx == sum(mean(mtcars$mpg))))
+})
+
+test_that("mutate.rowwise handles factors (#886)", {
+  res <- data.frame(processed=c("foo", "bar")) %>%
+    rowwise() %>%
+    mutate(processed_trafo=paste("test", processed))
+  expect_equal( res$processed_trafo, c("test foo", "test bar"))
+})
+
+test_that("setting first column to NULL with mutate works (#1329)", {
+    df <- data.frame(x = 1:10, y = 1:10)
+    expect_equal( mutate(df, x=NULL), select(df,-x) )
+    expect_equal( mutate(df, y=NULL), select(df,-y) )
+
+    gdf <- group_by(df, y)
+    expect_equal( select(gdf, -x), mutate(gdf, x = NULL) )
+})
+
+test_that("mutate handles the all NA case (#958)", {
+  x <- rep(c("Bob", "Jane"), each = 36)
+  y <- rep(rep(c("A", "B", "C"), each = 12), 2)
+  day <- rep(rep(1:12, 3), 2)
+  values <- rep(rep(c(10, 11, 30, 12, 13, 14, 15, 16, 17, 18, 19, 20), 3), 2)
+
+  df <- data.frame(x = x, y = y, day = day, values = values)
+  df$values[1:12] <- NA
+
+  res <- df %>%
+    group_by(x, y) %>%
+    mutate(max.sum = day[which.max(values)[1]]) %>%
+    mutate(adjusted_values = ifelse(day < max.sum, 30, values))
+  expect_true( all(is.na( res$adjusted_values[1:12] )))
+
+})
